function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) {
    try {
        var info = gen[key](arg);
        var value = info.value;
    } catch (error) {
        reject(error);
        return;
    }
    if (info.done) {
        resolve(value);
    } else {
        Promise.resolve(value).then(_next, _throw);
    }
}
function _async_to_generator(fn) {
    return function() {
        var self = this, args = arguments;
        return new Promise(function(resolve, reject) {
            var gen = fn.apply(self, args);
            function _next(value) {
                asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value);
            }
            function _throw(err) {
                asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err);
            }
            _next(undefined);
        });
    };
}
import { browserslistToTargets } from 'lightningcss';
import PATH from 'path';
import browserslist from 'browserslist';
import { camelCase } from 'camel-case';
export const codeWithSourceMap = (code, map)=>{
    return code + '/*# sourceMappingURL=data:application/json;base64,' + Buffer.from(map).toString('base64') + ' */';
};
export const cssExportsToJs = (exports, entryFile)=>{
    const keys = Object.keys(exports).sort();
    const values = keys.map((key)=>exports[key]);
    let variablesCode = '';
    const exportCode = [];
    const defaultCode = [];
    keys.forEach((key, index)=>{
        const clsVar = `s_${camelCase(key)}`;
        const clsSelector = values[index].name;
        variablesCode += `var ${clsVar} = "${clsSelector}";\n`;
        exportCode.push(`exports['${key}'] = ${clsVar};`);
        defaultCode.push(`'${key}':${clsVar}`);
    });
    const code = `${variablesCode};\n${exportCode.join('\n')};\nmodule.exports = {${defaultCode.join(',')}};\n`;
    if (entryFile) {
        return `require('${entryFile}');\n${code}`;
    }
    return code;
};
/**
 * 从路径中解析文件路径和 query
 */ export const parsePath = (path)=>{
    const queryIndex = path.indexOf('?');
    if (queryIndex === -1) {
        return {
            path: path,
            query: ''
        };
    } else {
        return {
            path: path.slice(0, queryIndex),
            query: path.slice(queryIndex + 1)
        };
    }
};
export function resolvePath(args, build) {
    return _resolvePath.apply(this, arguments);
}
function _resolvePath() {
    _resolvePath = _async_to_generator(function*(args, build) {
        const { path, query } = parsePath(args.path);
        let absolutePath = path;
        if (!PATH.isAbsolute(absolutePath)) {
            const result = yield build.resolve(absolutePath, {
                resolveDir: args.resolveDir,
                importer: args.importer,
                kind: args.kind
            });
            absolutePath = result.path;
        }
        return {
            path: absolutePath,
            query
        };
    });
    return _resolvePath.apply(this, arguments);
}
export const generateTargets = (queries)=>{
    return browserslistToTargets(browserslist(queries));
};
export const replaceExtension = (file, ext)=>{
    const extName = PATH.extname(file);
    return file.slice(0, file.length - extName.length) + ext;
};
